@@ -1,4 +1,4 @@
-/*  ocrad - Optical Character Recognition program
+/*  GNU OCRAD - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -231,59 +231,145 @@ Bitmap::Bitmap( const Bitmap & source, type t ) throw( Error )
   }
 
 
-int Bitmap::find_columns( bool verbose ) throw()
+int Bitmap::analyse_layout( int layout_level ) throw()
   {
-  if( _rectangle_list.size() != 0 )
-    internal_error( "find_columns called twice" );
+  if( _rectangle_vector.size() != 0 )
+    internal_error( "analyse_layout called twice" );
 
-  const int colmin = ( _width / 10 ) + 100;
-  const int gapmin = ( _width / 100 ) + 10;
-  std::vector< int > h_outline;
-  h_outline.resize( _width );
+  Rectangle r( 0, 0, _width - 1, _height - 1 );
+  _rectangle_vector.push_back( r );
+  if( layout_level <= 0 ) return _rectangle_vector.size();
 
-  int total_dots = 0;
-  for( int col = 0; col < _width; ++col )
+  find_columns();
+  if( layout_level <= 1 ) return _rectangle_vector.size();
+
+  while( find_rows() ) find_columns();
+  return _rectangle_vector.size();
+  }
+
+
+bool Bitmap::find_columns() throw()
+  {
+  bool split = false;
+  for( int rindex = 0; rindex < rectangles(); )
     {
-    h_outline[col] = 0;
-    for( int row = 0; row < _height; ++row )
-      if( data[row][col] ) ++h_outline[col];
-    total_dots += h_outline[col];
+    int columns = 0;
+    const Rectangle & r = _rectangle_vector[rindex];
+    const int colmin = ( r.width() / 10 ) + 100;
+    const int gapmin = ( r.width() / 100 ) + 10;
+    std::vector< int > h_outline;
+    h_outline.resize( r.width() );
+
+    int total_dots = 0;
+    for( int col = 0; col < r.width(); ++col )
+      {
+      h_outline[col] = 0;
+      for( int row = 0; row < r.height(); ++row )
+        if( data[r.top()+row][r.left()+col] ) ++h_outline[col];
+      total_dots += h_outline[col];
+      }
+    int threshold_col = ( total_dots / ( r.width() * 4 ) ) + 6;
+    int threshold_gap = ( total_dots / ( r.width() * 20 ) ) + 1;
+
+    int left, right, col_width, gap_width;
+    for( left = 0; left < r.width(); ++left )	// cut left border
+      if( h_outline[left] >= threshold_gap ) break;
+    right = left + 1;
+    left = ( left >= gapmin ) ? left / 2 : 0;
+
+    while( true )
+      {
+      const Rectangle & r = _rectangle_vector[rindex];
+      for( col_width = 0; right < r.width(); ++right )  // look for text
+        {
+        if( h_outline[right] > threshold_col ) ++col_width;
+        else if( col_width < colmin ) col_width = 0;
+        else break;
+        }
+      for( gap_width = 0; right < r.width(); ++right )  // look for gap
+        {
+        if( h_outline[right] < threshold_gap ) ++gap_width;
+        else if( gap_width < gapmin ) gap_width = 0;
+        else break;
+        }
+      if( right >= r.width() )
+        {
+        if( columns == 0 ) break;
+        ++columns;
+        Rectangle r1( r.left() + left, r.top(), r.right(), r.bottom() );
+        _rectangle_vector.insert( _rectangle_vector.begin() + rindex + columns, r1 );
+        break;
+        }
+      ++columns;
+      Rectangle r1( r.left() + left, r.top(), r.left() + right - ( gap_width / 2 ), r.bottom() );
+      _rectangle_vector.insert( _rectangle_vector.begin() + rindex + columns, r1 );
+      left = right - ( gap_width / 2 );
+      }
+    if( columns > 0 )
+      { _rectangle_vector.erase( _rectangle_vector.begin() + rindex );
+      rindex += columns; split = true; }
+    else ++rindex;
     }
-  int threshold_col = ( total_dots / ( _width * 4 ) ) + 6;
-  int threshold_gap = ( total_dots / ( _width * 20 ) ) + 1;
+  return split;
+  }
 
-  int left, right, col_width, gap_width;
-  for( left = 0; left < _width; ++left )	// cut left border
-    if( h_outline[left] >= threshold_gap ) break;
-  right = left + 1;
-  left = ( left >= gapmin ) ? left / 2 : 0;
 
-  while( true )
+bool Bitmap::find_rows() throw()
+  {
+  bool split = false;
+  for( int rindex = 0; rindex < rectangles(); ++rindex )
     {
-    for( col_width = 0; right < _width; ++right )  // look for text
+    Rectangle & r = _rectangle_vector[rindex];
+    const int rowmin = ( _height > 1000 ) ? _height / 10 : 100;
+    const int gapmin = ( _height > 2000 ) ? _height / 100 : 20;
+    if( r.height() < ( 2 * rowmin ) + gapmin ) continue;
+    std::vector< int > v_outline;
+    v_outline.resize( r.height() );
+
+    int total_dots = 0;
+    for( int row = 0; row < r.height(); ++row )
       {
-      if( h_outline[right] > threshold_col ) ++col_width;
-      else if( col_width < colmin ) col_width = 0;
-      else break;
+      v_outline[row] = 0;
+      for( int col = 0; col < r.width(); ++col )
+        if( data[r.top()+row][r.left()+col] ) ++v_outline[row];
+      total_dots += v_outline[row];
       }
-    for( gap_width = 0; right < _width; ++right )  // look for gap
+//    int threshold_row = ( total_dots / ( r.height() * 4 ) ) + 6;
+    int threshold_gap = ( total_dots / ( r.height() * 20 ) ) + 1;
+
+    int top, bottom;
+    for( top = 0; top < r.height(); ++top )	// cut top border
+      if( v_outline[top] > threshold_gap ) break;
+    for( bottom = r.height() - 1; bottom > top; --bottom ) // cut bottom border
+      if( v_outline[bottom] > threshold_gap ) break;
+    if( bottom - top < 2 * rowmin ) continue;
+
+    int t, b, best_top = 0, best_bottom = 0, gaps = 0;
+    for( t = top + gapmin; t < bottom; ++t )  // look for top of gap
+      if( v_outline[t] < threshold_gap )
+        {
+        ++gaps;
+        for( b = t + 1; b < bottom - rowmin; ++b)  // look for bottom of gap
+          if( v_outline[b] > threshold_gap ) break;
+        if( b - t > best_bottom - best_top )
+          { best_top = t; best_bottom = b; }
+        t = b;
+        }
+    if( gaps == 0 )
       {
-      if( h_outline[right] < threshold_gap ) ++gap_width;
-      else if( gap_width < gapmin ) gap_width = 0;
-      else break;
+      _rectangle_vector.erase( _rectangle_vector.begin() + rindex );
+      --rindex; continue;
       }
-    if( right >= _width )
+    if( best_bottom - best_top >= gapmin )
       {
-      Rectangle r( left, 0, _width - 1, _height - 1 );
-      _rectangle_list.push_back( r ); break;
+      int mid = r.top() + ( ( best_top + best_bottom ) / 2 );
+      Rectangle r1( r.left(), mid + 1, r.right(), r.bottom() );
+      r.bottom( mid ); ++rindex;
+      _rectangle_vector.insert( _rectangle_vector.begin() + rindex, r1 );
+      split = true;
       }
-    Rectangle r( left, 0, right - ( gap_width / 2 ), _height - 1 );
-    _rectangle_list.push_back( r );
-    left = right - ( gap_width / 2 );
     }
-  if( verbose )
-    fprintf( stderr, "number of text blocks %d\n", _rectangle_list.size() );
-  return _rectangle_list.size();
+  return split;
   }
 
 
