@@ -1,5 +1,19 @@
-/*  ocrad - Optical Character Recognition program
+/*  GNU OCRAD - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
 // Only "P1" (pbm) and "P4" (pbm RAWBITS) file formats are recognized.
@@ -9,7 +23,11 @@ class Bitmap
   int _width;
   int _height;
   std::vector< std::vector< bool > > data;
-  std::vector< Rectangle > _rectangle_list;
+  std::vector< Rectangle > _rectangle_vector;
+
+  bool find_columns() throw();
+  bool find_rows() throw();
+
 public:
   struct Error
     {
@@ -31,9 +49,10 @@ public:
   int height() const throw() { return _height; }
   bool get_bit( int row, int col ) const throw() { return data[row][col]; }
 
-  const std::vector< Rectangle > & rectangle_list() const throw()
-    { return _rectangle_list; }
-  int find_columns( bool verbose = false ) throw();
+  const std::vector< Rectangle > & rectangle_vector() const throw()
+    { return _rectangle_vector; }
+  int rectangles() const throw() { return _rectangle_vector.size(); }
+  int analyse_layout( int layout_level ) throw();
   int horizontalify( bool verbose = false ) throw();
   void save( FILE *, char filetype = '4' ) const throw();
 
