@@ -1,4 +1,4 @@
-/*  ocrad - Optical Character Recognition program
+/*  GNU OCRAD - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -152,9 +152,9 @@ bool Block::escape_bottom( int row, int col ) const throw()
   if( bm.id( row, col ) == _id ) return false;
   int l, r;
 
-  for( l = col; l > left(); --l )
+  for( l = col; l > left() + 1; --l )
     if( bm.id( row, l - 1 ) == _id ) break;
-  for( r = col; r < right(); ++r )
+  for( r = col; r < right() - 1; ++r )
     if( bm.id( row, r + 1 ) == _id ) break;
   while( l <= r && ++row <= bottom() )
     {
@@ -325,7 +325,9 @@ bool Block::top_hook( int *hdiff ) const throw()
 
   if( black_section != 2 ) return false;
   if( escape_top( row, lcol + 1 ) ) return false;
-  *hdiff = follow_bottom( row, lcol ) - follow_bottom( row, rcol );
+  int lrow = follow_bottom( row, lcol ), rrow = follow_bottom( row, rcol );
+  if( lrow <= row || rrow <= row ) return false;
+  *hdiff = lrow - rrow;
   return true;
   }
 
