@@ -1,4 +1,4 @@
-/*  ocrad - Optical Character Recognition program
+/*  GNU OCRAD - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -26,12 +26,13 @@
 #include "blockmap.h"
 
 
-void Blockmap::create_block( const Rectangle & r, int id ) throw()
+Block * Blockmap::create_block( const Rectangle & r, int id ) throw()
   {
-  Block b( r, *this, id );
-  _block_list.push_back( b );
+  _block_list.push_back( Block( r, *this, id ) );
+  return &_block_list.back();
   }
 
+
 void Blockmap::delete_block( int id ) throw()
   {
   std::list< Block >::reverse_iterator p = _block_list.rbegin();
@@ -47,9 +48,10 @@ void Blockmap::delete_block( int id ) throw()
   internal_error( "delete_block, lost block" );
   }
 
-void Blockmap::join_blocks( int id1, int id2 ) throw()
+
+Block * Blockmap::join_blocks( int id1, int id2 ) throw()
   {
-  if( id1 > id2 )
+  if( abs( id1 ) > abs( id2 ) )
     { int temp = id1; id1 = id2; id2 = temp; }
 
   std::list< Block >::reverse_iterator p1 = _block_list.rbegin();
@@ -62,123 +64,75 @@ void Blockmap::join_blocks( int id1, int id2 ) throw()
   for( int row = p2->top(); row <= p2->bottom(); ++row )
     for( int col = p2->left(); col <= p2->right(); ++col )
       if( data[row][col] == id2 ) data[row][col] = id1;
-  p1->add_point( p2->top(), p2->left() );
-  p1->add_point( p2->bottom(), p2->right() );
+  p1->add_rectangle( *p2 );
   _block_list.erase( --(p2.base()) );
+  return &(*p1);
   }
 
-void Blockmap::add_point_to_block( int row, int col, int id ) throw()
+
+Block * Blockmap::add_point_to_block( int row, int col, int id ) throw()
   {
   std::list< Block >::reverse_iterator p = _block_list.rbegin();
   for( ; p != _block_list.rend(); ++p )
-    if( id == p->id() ) { p->add_point( row, col ); return; }
+    if( id == p->id() ) { p->add_point( row, col ); return &(*p); }
 
   internal_error( "add_point_to_block, lost block" );
   }
 
 
-int Blockmap::left_neighbor( const Rectangle & r , int distance )
-							const throw()
-  {
-  int limit = distance >= r.left() ? 0 : r.left() - distance;
-
-  for( int col = r.left(); col > limit; )
-    {
-    --col;
-    for( int row = r.top(); row <= r.bottom(); ++row )
-      if( data[row][col] != 0 ) return data[row][col];
-    }
-  return 0;
-  }
-
-int Blockmap::top_neighbor( const Rectangle & r , int distance )
-								const throw()
-  {
-  int limit = distance >= r.top() ? 0 : r.top() - distance;
-
-  for( int row = r.top(); row > limit; )
-    {
-    --row;
-    for( int col = r.left(); col <= r.right(); ++col )
-      if( data[row][col] != 0 ) return data[row][col];
-    }
-  return 0;
-  }
-
-int Blockmap::right_neighbor( const Rectangle & r , int distance )
-								const throw()
-  {
-  int limit = r.right() + distance + 1;
-  if( limit > _width ) limit = _width;
-
-  for( int col = r.right() + 1; col < limit; ++col )
-    for( int row = r.top(); row <= r.bottom(); ++row )
-      if( data[row][col] != 0 ) return data[row][col];
-
-  return 0;
-  }
-
-int Blockmap::bottom_neighbor( const Rectangle & r , int distance )
-								const throw()
-  {
-  int limit = r.bottom() + distance + 1;
-  if( limit > _height ) limit = _height;
-
-  for( int row = r.bottom() + 1; row < limit; ++row )
-    for( int col = r.left(); col <= r.right(); ++col )
-      if( data[row][col] != 0 ) return data[row][col];
-
-  return 0;
-  }
-
-
 void delete_small_blocks( std::list< Block > & block_list ) throw()
   {
-  std::list< Block >::iterator p1 = block_list.begin();
-  while( p1 != block_list.end() )
+  std::list< Block >::iterator p = block_list.begin();
+  while( p != block_list.end() )
     {
-    std::list< Block >::iterator next = p1;
+    std::list< Block >::iterator next = p;
     ++next;
-    if( p1->height() <= 2 && p1->width() <= 2 /*&& p1->id() > 0*/ )
-      block_list.erase( p1 );
-    p1 = next;
+    if( p->height() <= 4 && p->width() <= 4 &&
+        ( ( p->height() <= 2 && p->width() <= 2 ) || p->area() <= 4 ) )
+      block_list.erase( p );
+    p = next;
     }
   }
 
-/*
-void hierarchize_blocks( std::list< Block > & block_list ) throw()
+
+void delete_white_blocks( std::list< Block > & block_list ) throw()
   {
-  std::list< Block >::iterator p1 = block_list.begin();
-  for( ; p1 != block_list.end(); ++p1 )
+  std::list< Block >::iterator p = block_list.begin();
+  while( p != block_list.end() )
     {
-    std::list< Block >::iterator p2 = block_list.begin();
-    while( p2 != block_list.end() )
-      {
-      std::list< Block >::iterator next = p2;
-      ++next;
-      if( p1 != p2 && p1->includes( *p2 ) )
-        {
-        int id = p2->id();
-        p1->_block_list.splice( p1->_block_list.end(), block_list, p2 );
-        if( id > 0 ) hierarchize_blocks( p1->_block_list );
-        }
-      p2 = next;
-      }
+    std::list< Block >::iterator next = p;
+    ++next;
+    if( p->id() < 0 ) block_list.erase( p );
+    p = next;
     }
   }
-*/
+
+
+void delete_wide_blocks( std::list< Block > & block_list, int width ) throw()
+  {
+  std::list< Block >::iterator p = block_list.begin();
+  while( p != block_list.end() )
+    {
+    std::list< Block >::iterator next = p;
+    ++next;
+    if( 2 * p->width() > width ) block_list.erase( p );
+    p = next;
+    }
+  }
+
 
 void hierarchize_blocks( std::list< Block > & block_list ) throw()
   {
   std::list< Block >::iterator p1 = block_list.begin();
   for( ; p1 != block_list.end(); ++p1 ) if( p1->id() > 0 )
     {
-    std::list< Block >::iterator p2 = block_list.begin();
+    std::list< Block >::iterator p2 = p1; ++p2;
     while( p2 != block_list.end() )
       {
+      if( p2->top() >= p1->bottom() ) break;
       std::list< Block >::iterator next = p2;
       ++next;
-      if( p1 != p2 && p1->includes( *p2 ) )
+      if( p1->includes( *p2 ) )
         {
         if( p2->id() < 0 )
           p1->_block_list.splice( p1->_block_list.end(), block_list, p2 );
@@ -211,79 +165,115 @@ void order_blocks( std::list< Block > & block_list ) throw()
   }
 
 
+void remove_top_bottom_noise( std::list< Block > & block_list ) throw()
+  {
+  std::list< Block >::iterator p = block_list.begin();
+  for( ; p != block_list.end(); ++p )
+    {
+    Block & b = *p;
+    if( b.height() <= 10 ) continue;
+
+    int c = 0;
+    for( int col = b.left(); col <= b.right(); ++col )
+      if( b.id( b.top(), col ) == b.id() && ++c > 1 ) break;
+    if( c <= 1 ) b.top( b.top() + 1 );
+
+    c = 0;
+    for( int col = b.left(); col <= b.right(); ++col )
+      if( b.id( b.bottom(), col ) == b.id() && ++c > 1 ) break;
+    if( c <= 1 ) b.bottom( b.bottom() - 1 );
+    }
+  }
+
+
 int generate_black_id() { static int next = 0; return ++next; }
 
 int generate_white_id() { static int next = 0; return --next; }
 
-Blockmap::Blockmap( const Bitmap & page_image, int column, int debug_level )
+Blockmap::Blockmap( const Bitmap & page_image, int rindex, int debug_level )
                                                                      throw()
   {
-  if( column < 0 ||
-      column >= static_cast<int>(page_image.rectangle_list().size()) )
-    internal_error( "Blockmap, column out of range" );
+  if( rindex < 0 ||
+      rindex >= static_cast<int>(page_image.rectangle_vector().size()) )
+    internal_error( "Blockmap, text rectangle index out of range" );
 
-  int row0 = page_image.rectangle_list()[column].top();
-  int col0 = page_image.rectangle_list()[column].left();
-  _height =  page_image.rectangle_list()[column].height();
-  _width =   page_image.rectangle_list()[column].width();
+  const Rectangle & r = page_image.rectangle_vector()[rindex];
+  int row0 = r.top();
+  int col0 = r.left();
+  _height =  r.height();
+  _width =   r.width();
   data.resize( _height );
 
   for( int row = 0; row < _height; ++row )
+    { data[row].resize( _width ); data[row][0] = 0; }
+  for( int col = 0; col < _width; ++col ) data[0][col] = 0;
+
+  for( int row = 1; row < _height; ++row )
     {
-    data[row].resize( _width );
-    for( int col = 0; col < _width; ++col )
+    Block *pb = 0, *pw = 0;
+    for( int col = 1; col < _width; ++col )
       {
       if( page_image.get_bit( row0 + row, col0 + col ) )	// black point
         {
-        if( row > 0 && data[row-1][col] > 0 )
+        if( data[row-1][col] > 0 )
           {
-          data[row][col] = data[row-1][col];
-          add_point_to_block( row, col, data[row][col] );
-          if( col > 0 && data[row][col-1] > 0 && data[row][col-1] != data[row-1][col] )
-            join_blocks( data[row-1][col], data[row][col-1] );
+          int id = data[row][col] = data[row-1][col];
+          if( pb && pb->id() == id ) pb->add_point( row, col );
+          else pb = add_point_to_block( row, col, id );
+          if( data[row][col-1] > 0 && data[row][col-1] != id )
+            pb = join_blocks( data[row][col-1], id );
           }
-        else if( col > 0 && data[row][col-1] > 0 )
+        else if( data[row][col-1] > 0 )
           {
-          data[row][col] = data[row][col-1];
-          add_point_to_block( row, col, data[row][col] );
+          int id = data[row][col] = data[row][col-1];
+          if( pb && pb->id() == id ) pb->add_point( row, col );
+          else pb = add_point_to_block( row, col, id );
           }
         else
           {
-          data[row][col] = generate_black_id();
-          create_block( Rectangle( col, row, col, row ), data[row][col] );
+          int id = data[row][col] = generate_black_id();
+          pb = create_block( Rectangle( col, row, col, row ), id );
           }
         }
       else						// white point
         {
-        if( row == 0 || col == 0 ) data[row][col] = 0;
-        else if( data[row-1][col] == 0 || data[row][col-1] == 0 )
+        if( data[row][col-1] == 0 )
           {
-          data[row][col] = 0;
+          data[row][col] = 0; pw = 0;
           if( data[row-1][col] < 0 ) delete_block( data[row-1][col] );
-          else if( data[row][col-1] < 0 ) delete_block( data[row][col-1] );
+          }
+        else if( data[row-1][col] == 0 )
+          {
+          data[row][col] = 0; pw = 0;
+          if( data[row][col-1] < 0 ) delete_block( data[row][col-1] );
           }
         else if( data[row-1][col] < 0 )
           {
-          data[row][col] = data[row-1][col];
-          add_point_to_block( row, col, data[row][col] );
-          if( data[row][col-1] < 0 && data[row][col-1] != data[row-1][col] )
-            join_blocks( data[row-1][col], data[row][col-1] );
+          int id = data[row][col] = data[row-1][col];
+          if( pw && pw->id() == id ) pw->add_point( row, col );
+          else pw = add_point_to_block( row, col, id );
+          if( data[row][col-1] < 0 && data[row][col-1] != id )
+            pw = join_blocks( data[row][col-1], id );
           }
         else if( data[row][col-1] < 0 )
           {
-          data[row][col] = data[row][col-1];
-          add_point_to_block( row, col, data[row][col] );
+          int id = data[row][col] = data[row][col-1];
+          if( pw && pw->id() == id ) pw->add_point( row, col );
+          else pw = add_point_to_block( row, col, id );
           }
         else
           {
-          data[row][col] = generate_white_id();
-          create_block( Rectangle( col, row, col, row ), data[row][col] );
+          int id = data[row][col] = generate_white_id();
+          pw = create_block( Rectangle( col, row, col, row ), id );
           }
         }
       }
     }
   delete_small_blocks( _block_list );
+  delete_wide_blocks( _block_list, _width );
   if( debug_level <= 99 ) hierarchize_blocks( _block_list );
+  delete_white_blocks( _block_list );
+  remove_top_bottom_noise( _block_list );
   if( debug_level <= 97 ) order_blocks( _block_list );
   }
 
