@@ -1,4 +1,4 @@
-/*  ocrad - Optical Character Recognition program
+/*  GNU OCRAD - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -146,3 +146,15 @@ void Character::print( FILE * outfile, bool graph, bool recursive )
     else fputc( '_', outfile );
     }
   }
+
+
+void Character::xprint( FILE * exportfile ) const throw()
+  {
+  fprintf( exportfile, "%d %d %d %d;%d",
+           left(), top(), width(), height(), guesses() );
+
+  for( int i = 0; i < guesses(); ++i )
+    fprintf( exportfile, ",'%c'%d", _guess[i].ch, _guess[i].value );
+
+  fprintf( exportfile, "\n" );
+  }
