@@ -1,5 +1,19 @@
-/*  ocrad - Optical Character Recognition program
+/*  GNU OCRAD - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
 class Character : public Rectangle
@@ -20,7 +34,7 @@ class Character : public Rectangle
     void recognize110( int charbox_vcenter ) throw();
     void recognize111() throw();
     void recognize112() throw();
-  void recognize12() throw();
+  void recognize12( int charbox_vcenter ) throw();
   void recognize13() throw();
 
 public:
@@ -52,5 +66,7 @@ public:
 
   void join( Character & c ) throw();
   void print( FILE * outfile, bool graph, bool recursive ) const throw();
+  void xprint( FILE * exportfile ) const throw();
+
   void recognize1( int charbox_vcenter ) throw();
   };
