@@ -1,4 +1,4 @@
-/*  ocrad - Optical Character Recognition program
+/*  GNU OCRAD - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -31,8 +31,7 @@ Features::Features( const Block & b ) throw()
   : _block( &b ), _blockmap( b.blockmap() ), _hbars( -1 ), _vbars( -1 ),
     hscan_valid( false ), vscan_valid( false ), lp( b, Profile::left ),
     tp( b, Profile::top ), rp( b, Profile::right ), bp( b, Profile::bottom ),
-    hp( b, Profile::height ), wp( b, Profile::width ),
-    zhp( b, Profile::zheight ), zwp( b, Profile::zwidth )
+    hp( b, Profile::height ), wp( b, Profile::width )
   {}
 
 
@@ -60,7 +59,7 @@ int Features::hbars() const throw()
         {
         case 0: if( maxcount > limit )
                   { state = 1; begin = row; l = lt; r = rt; }
-                break;
+                else break;
         case 1: if( maxcount > limit )
                   {
                   if( lt < l ) l = lt;
@@ -83,12 +82,13 @@ int Features::hbars() const throw()
   }
 
 
-int Features::vbars() const throw()
+int Features::vbars() const throw()	// FIXME small gaps not detected
   {
   if( _vbars < 0 )
     {
     const Block & b = *_block;
-    int state = 0, begin = 0, limit = b.height() - 3;
+    int state = 0, begin = 0, limit = b.height();
+    limit -= ( b.height() < 40 ) ? 3 : b.height() / 10;
     _vbars = 0;
 
     for( int col = b.left(); col <= b.right(); ++col )
@@ -198,7 +198,6 @@ unsigned char Features::test_235Esz() const throw()
   for( col = b.hcenter() - 1; col <= b.hcenter() + 1; ++col )
     {
     bool prev_black = false;
-    black_section = 0;
     for( int row = b.top(); row <= b.bottom(); ++row )
       {
       bool black = ( b.id( row, col ) == b.id() );
@@ -212,10 +211,12 @@ unsigned char Features::test_235Esz() const throw()
           else if( black_section == 2 ) lrow2 = row; }
       prev_black = black;
       }
-    if( black_section == 3 ) break;
+    if( black_section == 3 && lrow1 <= b.vpos(30) && urow3 >= b.vpos(70) )
+      break;
+    black_section = 0;
     }
 
-  if( black_section != 3 || lrow1 >= b.vcenter() ) return 0;
+  if( black_section != 3 ) return 0;
   if( b.escape_left( lrow2, col ) )
     {
     if( b.escape_left( urow2, col ) )
@@ -266,21 +267,33 @@ unsigned char Features::test_49egpq() const throw()
   const Block & b = *_block;
   const Block & h = b.block_list().front();		// hole
 
-  int row = b.seek_bottom( h.bottom(), h.hcenter(), false ) + 1;
+  int col = h.hcenter();
+  int row = b.seek_bottom( h.bottom(), col, false ) + 1;
+  if( row >= b.vpos(90) )
+    { col = h.left(); row = b.seek_bottom( h.bottom(), col, false ) + 1; }
   if( row >= b.bottom() ) return 0;
-  if( b.escape_right( row, h.hcenter() ) )
-    { if( lp.isconvex() || bp.isconvex() ) return 'e'; else return 'p'; }
-  if( b.escape_left( row, h.hcenter() ) )
+
+  if( b.escape_right( row, col ) )
     {
+    if( lp.ispit() && b.seek_bottom( row, h.right() ) < b.bottom() )
+      return 'e';
+    else return 'p';
+    }
+
+  if( b.escape_left( row, col ) )
+    {
+    Profile hlp( h, Profile::left );
     Profile htp( h, Profile::top );
-    if( vbars() == 1 && vbar(0).hcenter() > b.hcenter() && htp.decreasing() )
+    if( vbars() == 1 && vbar(0).hcenter() > b.hcenter() &&
+        hlp.decreasing() && htp.decreasing() )
       return '4';
     if( rp.isconvex() && tp.isconvex() && rp.minima() == 1 )
       return '9';
     int hdiff;
+    if( b.top_hook( &hdiff ) && hdiff > 0 ) return 's';
     if( b.bottom_hook( &hdiff ) && hdiff > 0 ) return 'g';
-    if( row > b.bottom() - ( b.height() / 4 ) ) return 'Q';
-    if( b.seek_bottom( row, h.hcenter() ) < b.bottom() &&
+    if( row > b.vpos(80) ) return 'Q';
+    if( b.seek_bottom( row, col ) < b.bottom() &&
         rp.increasing( row - b.top() ) ) return 'g';
     return 'q';
     }
@@ -295,22 +308,33 @@ unsigned char Features::test_6abd() const throw()
   const Block & b = *_block;
   const Block & h = b.block_list().front();		// hole
 
-  int row = b.seek_top( h.top(), h.hcenter(), false ) - 1;
+  int col = h.hcenter();
+  int row = b.seek_top( h.top(), col, false ) - 1;
+  if( row <= b.top() )
+    { col = h.right(); row = b.seek_top( h.top(), col, false ) - 1; }
   if( row <= b.top() ) return 0;
-  bool oacute1 = ( b.seek_right( h.top() - ( b.bottom() - h.bottom() ) - 1, h.right() ) >= b.right() );
+  int rcol = ( b.right() + h.right() ) / 2;
+  int urow = h.top() - ( b.bottom() - h.bottom() );
+  bool oacute1 = ( b.seek_right( urow - 1, h.right() ) >= b.right() );
 
-  if( b.escape_right( row, h.hcenter() ) )
+  if( b.escape_right( row, col ) )
     {
-    if( lp.isconvex() && bp.isconvex() ) return '6';
-    bool oacute2 = ( b.seek_left( b.top() + 1, b.right() + 1 ) > h.right() );
-    if( oacute1 && oacute2 ) return '';
-    int row2 = b.seek_top( h.top(), h.right(), false ) - 1;
-    row2 = b.seek_top( row2, h.right() );
+    bool oacute2 = ( b.seek_right( urow, b.left() ) > b.seek_right( h.top(), b.left() ) + 1 );
+//    if( !oacute2 && lp.isconvex() && bp.isconvex() ) return '6';
+    int row2 = b.seek_top( h.top(), rcol, false ) - 1;
+    row2 = b.seek_top( row2, rcol );
+    if( !oacute2 && lp.ispit() && bp.ispit() && row2 > b.top() ) return '6';
+    bool oacute3 = ( b.seek_left( b.top() + 1, b.right() + 1 ) >= rcol ||
+                     b.seek_right( h.top(), b.left() ) <= b.left() + 1 );
+    if( oacute1 && oacute2 && oacute3 ) return '';
     if( row2 <= b.top() ) return 'b'; else return 's';
     }
-  if( b.escape_left( row, h.hcenter() ) )
+
+  if( b.escape_left( row, col ) )
     {
-    if( b.seek_bottom( b.top() - 1, h.left() ) < row )
+    int row2 = b.seek_top( h.top(), h.left(), false ) - 1;
+    row2 = b.seek_top( row2, h.left() );
+    if( row2 > b.top() )
       { if( b.vcenter() < row ) return ''; else return 'a'; }
     if( oacute1 ) return ''; else return 'd';
     }
@@ -341,9 +365,10 @@ unsigned char Features::test_CEFIJLlT() const throw()
     }
 
   if( vbars() == 1 && vbar(0).width() >= 2 &&
-      vbar(0).width() * 3 < b.width() )
+      3 * vbar(0).width() <= b.width() + 2 )
     {
-    if( abs( vbar(0).hcenter() - b.hcenter() ) <= 1 )
+    if( abs( vbar(0).hcenter() - b.hcenter() ) <= 1 &&
+        abs( (vbar(0).left() - b.left()) - (b.right() - vbar(0).right()) ) <= 2 )
       {
       if( hbars() == 1 )
         {
@@ -351,16 +376,17 @@ unsigned char Features::test_CEFIJLlT() const throw()
         if( abs( hbar(0).vcenter() - b.vcenter() ) <= 1 ) return '+';
         }
       if( hbars() == 2 && hbar(0).top() <= b.top() + 1 &&
-          hbar(0).width() > hbar(1).width() )
+          3 * hbar(0).width() > 4 * hbar(1).width() )
         return 'T';
       }
     if( vbar(0).right() < b.hcenter() )
       {
       if( hbars() == 3 && hbar(0).top() <= b.top() + 1 &&
-          hbar(0).width() > hbar(1).width() )
+          hbar(0).width() + 1 >= hbar(1).width() &&
+          vbar(0).h_overlaps( hbar(1) ) )
         {
         if( similar( hbar(0).width(), hbar(2).width(), 10 ) &&
-            hbar(2).width() > hbar(1).width() )
+            hbar(2).width() >= hbar(1).width() )
           return 'E';
         if( hbar(0).width() > hbar(2).width() &&
             hbar(1).includes_vcenter( b ) )
@@ -375,12 +401,10 @@ unsigned char Features::test_CEFIJLlT() const throw()
         if( hbar(1).bottom() >= b.bottom() - 1 &&
             b.height() > b.width() &&
             hbar(1).width() > hbar(0).width() &&
-            abs( vbar(0).hcenter() - hbar(0).hcenter() ) <= 1 )
+            abs( vbar(0).hcenter() - hbar(0).hcenter() ) <= 1 &&
+            rp.iminimum() > rp.pos( 70 ) )
           return 'L';
         }
-      if( hbars() == 1 && hbar(0).bottom() >= b.bottom() - 1 &&
-          similar( hbar(0).width(), b.width(), 10 ) )
-        return 'L';
       }
     if( vbar(0).left() > b.hcenter() )
       {
@@ -389,6 +413,13 @@ unsigned char Features::test_CEFIJLlT() const throw()
         return '';
       }
     }
+
+  if( vbars() == 1 && vbar(0).width() >= 2 &&
+      vbar(0).width() * 2 < b.width() && vbar(0).right() < b.hcenter() &&
+      hbars() == 1 && hbar(0).bottom() >= b.bottom() - 1 &&
+      similar( hbar(0).width(), b.width(), 10 ) )
+    return 'L';
+
   if( vbars() == 1 && vbar(0).width() >= 2 )
     {
     if( 3 * b.height() > 4 * b.width() &&
@@ -400,11 +431,12 @@ unsigned char Features::test_CEFIJLlT() const throw()
         return '{';
       if( lp.istip() && rp.ispit() && similar( rp.iminimum(), rp.pos(50), 10 ) )
         return '}';
-      if( vbar(0).right() >= b.hpos( 70 ) ||
-          b.escape_top( b.vpos( 75 ), vbar(0).right() + 1 ) )
-        for( int i = vbar(0).left() - 1; i > b.left(); --i )
-          if( b.seek_bottom( b.vpos( 75 ), i ) < b.bottom() &&
-              bp[i-b.left()] <= 1 ) return 'l';
+      if( 2 * b.height() > 3 * b.width() )
+        if( vbar(0).right() >= b.hpos( 70 ) ||
+            b.escape_top( b.vpos( 75 ), vbar(0).right() + 1 ) )
+          for( int i = vbar(0).left() - 1; i > b.left(); --i )
+            if( b.seek_bottom( b.vpos( 75 ), i ) < b.bottom() &&
+                bp[i-b.left()] <= 1 ) return 'l';
       }
     if( vbar(0).left() <= b.left() + 1 && b.height() > 2 * b.width() &&
         rp.istip() )
@@ -417,8 +449,15 @@ unsigned char Features::test_CEFIJLlT() const throw()
         return 'C';
       return '[';
       }
-    if( vbar(0).right() >= b.right() - 1 && b.height() > 2 * b.width() &&
-        lp.istip() ) return ']';
+    if( vbar(0).right() >= b.right() - 1 && b.height() > 2 * b.width() )
+      {
+      if( lp.istip() )
+        {
+        if( 2 * vbar(0).width() <= wp.max() ) return ']';
+        if( b.height() >= 3 * b.width() ) return 'l';
+        }
+      if( lp.isvpit() && lp.iminimum() < lp.pos(40) ) return '1';
+      }
     }
   if( hbars() == 1 && abs( hbar(0).vcenter() - b.vcenter() ) <= 1 &&
       tp.isupit() && bp.isupit() ) return '+';
@@ -426,14 +465,112 @@ unsigned char Features::test_CEFIJLlT() const throw()
   }
 
 
-unsigned char Features::test_cG() const throw()
+unsigned char Features::test_c() const throw()
   {
   const Block & b = *_block;
 
   if( lp.isconvex() )
     {
-    int row = b.seek_bottom( b.vcenter(), b.hcenter() );
-    if( row < b.bottom() && b.escape_right( b.vcenter(), b.hcenter() ) &&
+    if( b.height() > 2 * b.width() ) return '(';
+    if( b.escape_right( b.vcenter(), b.hcenter() ) ) return 'c';
+    }
+  if( lp.ispit() && bp.ispit() && tp.ispit() &&
+      b.escape_right( b.vcenter(), b.hcenter() ) )
+    return 'c';
+  return 0;
+  }
+
+
+unsigned char Features::test_frst() const throw()
+  {
+  const Block & b = *_block;
+
+  if( tp.minima( b.height() / 4 ) != 1 || bp.minima( b.height() / 4 ) != 1 )
+    return 0;
+
+    {
+    int b_hdiff = 0, t_hdiff = 0;
+    if( b.bottom_hook( &b_hdiff ) )
+      {
+      if( -b_hdiff > b.height() / 2 )
+        {
+        if( b.height() >= 3 * wp.max() &&
+            ( hbars() == 0 || hbar(0).bottom() < b.vpos( 20 ) ) ) return 'l';
+        if( 3 * wp[1] < b.width() && hbars() >= 1 && hbars() <= 2 &&
+            hbar(0).top() >= b.vpos( 20 ) && hbar(0).bottom() < b.vcenter() &&
+            similar( hbar(0).width(), wp.max(), 10 ) ) return 't';
+        }
+      }
+    if( b.top_hook( &t_hdiff ) && t_hdiff > b.height() / 2 )
+      {
+      if( b.height() >= 2 * wp.max() &&
+          tp.iminimum() > tp.pos( 50 ) && bp.iminimum() < bp.pos( 50 ) )
+        return 'f';
+      if( b_hdiff > b.height() / 2 ) return 's';
+      }
+    }
+
+  if( vbars() != 1 || vbar(0).width() < 2 ) return 0;
+  if( vbar(0).hcenter() <= b.hcenter() )
+    {
+    if( similar( b.height(), b.width(), 40 ) )
+      {
+      int col = b.right() - rp[b.vcenter()-b.top()] + 2;
+      int row = b.seek_bottom( b.vcenter(), col );
+      if( row >= b.bottom() || b.escape_bottom( row - 1, col ) )
+        {
+        if( rp.minima() == 3 ) return 'f';
+        if( tp.minima( b.height() / 8 ) == 2 &&
+            bp.minima( b.height() / 8 ) == 2 ) return 'x';
+        int row = b.vpos(75);
+        int col = b.seek_right( row, b.hcenter(), false ) + 1;
+        if( b.seek_right( row, col ) >= b.right() )
+          { if( lp.isconvex() ) return 'c'; else  return 'r'; }
+        }
+      if( b.seek_right( row - 1, col ) < b.right() && lp.isctip() )
+        return 'x';
+      if( lp.isvpit() ) return 't';
+      }
+    else if( b.height() > b.width() && vbar(0).left() > b.left() &&
+             rp.minima() <= 2 )
+      {
+      int col = b.right() - rp[b.vcenter() - b.top()] + 1;
+      if( !b.escape_bottom( b.vcenter(), col ) )
+        { if( tp.ispit() && lp.iminimum() < lp.pos(40) ) return 't'; }
+      else if( 2 * wp.max() > b.width() )
+        {
+        if( rp.iminimum() < rp.pos(20) )
+          {
+          if( rp.increasing( rp.pos(20) ) || bp.increasing() ||
+              tp.minima( 1 ) == 2 ) return 'r';
+          else return 'f';
+          }
+        else return 't';
+        }
+      }
+    if( b.seek_bottom( b.vcenter(), b.hpos( 60 ) + 1 ) >= b.bottom() )
+      { if( rp.minima() == 2 ) return 'f'; else return 'r'; }
+    }
+  return 0;
+  }
+
+
+unsigned char Features::test_G() const throw()
+  {
+  const Block & b = *_block;
+
+  if( lp.isconvex() || lp.ispit() )
+    {
+    int col = 0, row = 0;
+    for( int i = rp.pos( 40 ); i <= rp.pos( 60 ); ++i )
+      if( rp[i] > col ) { col = rp[i]; row = i; }
+    if( col == 0 ) return 0;
+    row += b.top(); col = b.right() - col + 1;
+    if( col <= b.left() || col >= b.hcenter() ) return 0;
+
+    col = ( col + b.hcenter() ) / 2;
+    row = b.seek_bottom( row, col );
+    if( row < b.bottom() && b.escape_right( row, col ) &&
         !b.escape_bottom( row, b.hcenter() ) )
       {
       int lrow, urow;
@@ -450,64 +587,6 @@ unsigned char Features::test_cG() const throw()
           return 'G';
         }
       }
-    if( b.height() > 2 * b.width() ) return '(';
-    if( b.escape_right( b.vcenter(), b.hcenter() ) )
-      return 'c';
-    }
-  return 0;
-  }
-
-
-unsigned char Features::test_frt() const throw()
-  {
-  const Block & b = *_block;
-
-  if( tp.minima( b.height() / 4 ) != 1 || bp.minima( b.height() / 4 ) != 1 )
-    return 0;
-
-    {
-    int hdiff;
-    if( b.bottom_hook( &hdiff ) )
-      {
-      if( -hdiff > b.height() / 2 )
-        {
-        if( b.height() >= 3 * wp.max() &&
-            ( hbars() == 0 || hbar(0).bottom() < b.vpos( 20 ) ) ) return 'l';
-        if( 3 * wp[1] < b.width() && hbars() >= 1 && hbars() <= 2 &&
-            hbar(0).top() >= b.vpos( 20 ) && hbar(0).bottom() < b.vcenter() &&
-            similar( hbar(0).width(), wp.max(), 10 ) ) return 't';
-        }
-      }
-    if( b.height() >= 2 * wp.max() &&
-        b.top_hook( &hdiff ) && hdiff > b.height() / 2 &&
-        tp.iminimum() > tp.pos( 50 ) && bp.iminimum() < bp.pos( 50 ) )
-      return 'f';
-    }
-
-  if( vbars() != 1 || vbar(0).width() < 2 ) return 0;
-  if( vbar(0).hcenter() < b.hcenter() )
-    {
-    if( similar( b.height(), b.width(), 40 ) )
-      {
-      if( b.escape_bottom( b.vcenter(), b.right() - rp[b.vcenter() - b.top()] + 1 ) )
-        {
-        if( rp.minima() == 3 ) return 'f';
-        int row = b.bottom() - (b.height() / 4 );
-        int col = b.seek_right( row, b.hcenter(), false ) + 1;
-        col = b.seek_right( row, col );
-        if( col >= b.right() ) return 'r';
-        if( lp.isctip() ) return 'x';
-        }
-      if( lp.isvpit() ) return 't';
-      }
-    else if( vbar(0).left() > b.left() )
-      {
-      if( b.escape_bottom( b.vcenter(), b.right() - rp[b.vcenter() - b.top()] + 1 ) )
-        return 'f';
-      else return 't';
-      }
-    else if( b.seek_bottom( b.vcenter(), b.hcenter() ) >= b.bottom() )
-      { if( rp.minima() == 2 ) return 'f'; else return 'r'; }
     }
   return 0;
   }
@@ -519,29 +598,33 @@ unsigned char Features::test_HKMNUuvwYy() const throw()
 
   if( tp.minima( b.height() / 4 ) == 2 && tp.isctip() )
     {
-    if( bp.minima( b.height() / 5 ) == 1 )
+    if( b.height() >= 10 && bp.minima( b.height() / 5 ) == 1 )
       {
-      if( bp.isconvex() && bp.isvpit() ) return 'v';
+      int lg = lp[lp.samples()-2];
+      if( lg > 1 && bp.isvpit() && tp.minima( b.height()/2 ) == 2 &&
+          lp[lp.pos(75)] <= lg ) return 'v';
       int hdiff;
       if( b.bottom_hook( &hdiff ) )
         {
         if( abs( hdiff ) <= b.height() / 8 )
           {
           if( vscan()[b.height()/4] >= 3 ) return 'v';
-          if( bp.isconvex() )
-            { if( rp.increasing( rp.pos(75) ) ) return 'U'; else return 'u'; }
+          if( bp.isconvex() ) return 'u';
           }
         if( hdiff > b.height() / 2 ) return 'y';
         }
-      int lg = lp[lp.samples()-2];
       int rg = rp[rp.samples()-2];
       if( bp.ispit() && 7 * tp.range() < 4 * b.height() )
         if( ( similar( lg, rg, 20 ) &&
               5 * wp[wp.pos(98)] >= 4 * wp[wp.pos(70)] ) ||
             ( lg > 1 && lg < rg && !rp.increasing() ) ) return 'Y';
       if( 2 * lg < rg && lp.isctip() ) return 'y';
-      if( bp.isvpit() ) return 'v';
       if( b.escape_top( b.vcenter(), b.hcenter() ) ) return 'u';
+      if( lg < rg + 1 && !lp.increasing() && rp.increasing() &&
+          tp.minima( b.height()/2 ) == 1 ) return 'y';
+      if( lg > 1 && bp.ispit() && tp.minima( b.height()/2 ) == 2 ) return 'v';
+      if( lg <= 1 && 2 * ( b.width() - rg - lg ) < b.width() &&
+          rp.increasing() && tp.minima( b.height()/2 ) == 2 ) return 'v';
       return 0;
       }
     if( bp.minima( b.height() / 4 ) == 2 && bp.isctip() )
@@ -551,7 +634,8 @@ unsigned char Features::test_HKMNUuvwYy() const throw()
       int rg = rp[rp.pos(50)];
       if( similar( lg, rg, 80 ) && lg < b.width() / 4 && rg < b.width() / 4 )
         {
-        if( lp.increasing() && rp.increasing() ) return 'w';
+        if( lg > 1 && rg > 1 && lp.increasing() && rp.increasing() )
+          return 'w';
         if( hbars() == 1 && abs( hbar(0).vcenter() - b.vcenter() ) <= 1 &&
             abs( hbar(0).hcenter() - b.hcenter() ) <= 1 )
           return 'H';
@@ -577,20 +661,24 @@ unsigned char Features::test_hnw() const throw()
   int col = 0, row = 0;
   for( int i = bp.pos( 40 ); i <= bp.pos( 60 ); ++i )
     if( bp[i] > row ) { row = bp[i]; col = i; }
-  if( row == 0 ) return 0;
+  if( row < 2 ) return 0;
   row = b.bottom() - row + 1; col += b.left();
   if( row <= b.top() ) return 0;
+  //FIXME
+  { int c = col; col = b.seek_right( row, col ); if( col > c ) --col;
+  row = b.seek_top( row, col ); }
 
   if( tp.isconvex() && b.width() >= b.height() && lp.decreasing() ) return '^';
   int urow = b.seek_top( row - 1, col, false );
   if( urow > b.vpos( 20 ) )
     {
-    if( tp.minima( b.height() / 5 ) >= 2 ) return 'w';
+    if( tp.minima() >= 2 && tp.minima( b.height() / 5 ) >= 2 ) return 'w';
     if( tp.minima( b.height() / 5 ) == 1 )
       { if( rp.isctip( 66 ) ) return 'k'; else return 'h'; }
     }
   if( similar( b.height(), b.width(), 40 ) && row > b.vcenter() &&
-      tp.minima( b.height() / 5 ) == 2 ) return 'w';
+      urow < b.vcenter() && tp.minima( b.height() / 5 ) == 2 &&
+      bp.minima( urow + 1 ) == 3 ) return 'w';
   if( urow <= b.vpos( 20 ) && tp.minima( b.height() / 5 ) == 1 &&
       ( b.width() > b.height() || similar( b.height(), b.width(), 40 ) ) )
     return 'n';
@@ -625,7 +713,7 @@ bool Features::test_comma() const throw()
 
 // Recognizes single line, non-rectangular characters without holes.
 // '/<>\^`
-unsigned char Features::test_line() const throw()
+unsigned char Features::test_line( int charbox_vcenter ) const throw()
   {
   const Block & b = *_block;
   int slope1, slope2;
@@ -635,12 +723,17 @@ unsigned char Features::test_line() const throw()
     if( slope1 < 0 )
       {
       if( slope2 < 0 && bp.minima() == 2 ) return '^';
-      if( 4 * b.area() < b.height() * b.width() ) return '/';
+      if( b.v_includes( charbox_vcenter ) )
+        {
+        if( 4 * b.area() < b.height() * b.width() ) return '/';
+        else return 'l';
+        }
+      if( b.top() >= charbox_vcenter ) return ',';
       return '\'';
       }
     if( slope1 > 0 && slope2 < 0 )
       {
-      if( 4 * b.area() < b.height() * b.width() ) return '\\';
+      if( b.bottom() > charbox_vcenter ) return '\\';
       return '`';
       }
     }
@@ -653,10 +746,10 @@ unsigned char Features::test_line() const throw()
   }
 
 
-unsigned char Features::test_solid() const throw()
+unsigned char Features::test_solid( int charbox_vcenter ) const throw()
   {
   const Block & b = *_block;
-  if( b.block_list().size() > 0 ) return 0;
+  if( b.blocks() ) return 0;
   int inner_area, inner_space;
 
   if( b.width() >= 3 && b.height() >= 3 )
@@ -668,7 +761,7 @@ unsigned char Features::test_solid() const throw()
       int holes = 0;
       for( int col = b.left() + 1; col < b.right(); ++col )
         { if( b.id( row, col ) == b.id() ) ++inner_area; else ++holes; }
-      if( 4 * holes >= b.width() ) return 0;
+      if( 5 * holes >= b.width() ) return 0;
       }
     if( inner_area * 100 < inner_space * 75 ) return 0;
     }
@@ -677,14 +770,15 @@ unsigned char Features::test_solid() const throw()
   if( abs( b.height() - wp.max() ) <= 2 ||
       similar( b.height(), wp.max(), 20 ) ) return '.';
   if( inner_area * 100 < inner_space * 85 ) return 0;
-  if( b.width() >= 6 * b.height() ) return '_';
+  if( b.width() >= 6 * b.height() &&
+      b.top() - b.height() > charbox_vcenter ) return '_';
   if( b.width() > b.height() ) return '-';
-  if( b.height() >= 9 )
+  if( b.height() > 9 && b.width() <= 8 && b.v_includes( charbox_vcenter ) )
     {
     int l = 0, r = 0;
     for( int row = b.vcenter() - 1; row <= b.vcenter() + 1; ++row )
       { if( b.id( row, b.left() ) == b.id() ) ++l;
-      if( b.id( row, b.left() ) == b.id() ) ++r; }
+      if( b.id( row, b.right() ) == b.id() ) ++r; }
     if( l <= 1 && r <= 1 )
       {
       bool lt = false, lb = false, rt = false, rb = false;
@@ -697,8 +791,13 @@ unsigned char Features::test_solid() const throw()
       if( lt && lb && rb ) { if( rt ) return 'I'; else return 'l'; }
       }
     }
-  if( b.height() >= 5 * b.width() ) return '|';
-  if( b.height() > b.width() ) return '\'';
+  if( b.height() > b.width() )
+    {
+//    if( b.height() >= 5 * b.width() ) return '|';
+    if( b.top() > charbox_vcenter ) return ',';
+    if( b.bottom() <= charbox_vcenter ) return '\'';
+    return '|';
+    }
   return 0;
   }
 
@@ -709,7 +808,8 @@ unsigned char Features::test_misc() const throw()
 
   if( hbars() == 1 && hbar(0).top() <= b.top() + (b.height() / 10) &&
       5 * hbar(0).width() >= 4 * b.width() &&
-      rp.increasing( hbar(0).vcenter() - b.top() ) )
+      rp.increasing( hbar(0).vcenter() - b.top() ) &&
+      rp[hbar(0).bottom()-b.top()+2] - rp[hbar(0).bottom()-b.top()] < b.width() / 4 )
     return '7';
 
   if( b.height() > b.width() && rp.increasing( 1 ) &&
