@@ -1,4 +1,4 @@
-/*  ocrad - Optical Character Recognition program
+/*  GNU OCRAD - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -19,7 +19,7 @@
     Return values: 0 for a normal exit, 1 for environmental problems
     (file not found, invalid flags, I/O errors, etc), 2 to indicate a
     corrupt or invalid input file, 3 for an internal consistency error
-    (eg, bug) which caused 'ocrad' to panic.
+    (eg, bug) which caused OCRAD to panic.
 */
 
 #include <cctype>
@@ -36,13 +36,15 @@
 #include "blockmap.h"
 #include "character.h"
 #include "textline.h"
-#include "textpage.h"
+#include "textblock.h"
 
 
 void show_version()
   {
-  printf( "%s version %s\n", PROGRAM_NAME, PROGRAM_VERSION );
+  printf( "GNU %s version %s\n", PROGRAM_NAME, PROGRAM_VERSION );
   printf( "Copyright (C) %s Antonio Diaz.\n", PROGRAM_YEAR );
+  printf( "This program is free software; you may redistribute it under the terms of\n" );
+  printf( "the GNU General Public License.  This program has absolutely no warranty.\n" );
   }
 
 void show_error( char * msg )
@@ -53,59 +55,64 @@ void show_error( char * msg )
 
 void show_help()
   {
-  printf( "Usage: %s [options] [files]\n", PROGRAM_NAME );
-  printf( "Optical Character Recognition program\n" );
+  printf( "GNU %s, Optical Character Recognition program\n", PROGRAM_NAME );
   printf( "Reads pbm file(s), or standard input, and sends text to standard output\n" );
+  printf( "\nUsage: %s [options] [files]\n", PROGRAM_NAME );
   printf( "Options:\n");
   printf( "  -h, --help               display this help and exit\n");
   printf( "  -V, --version            output version information and exit\n");
   printf( "  -1, -4                   pbm output file type (debug)\n");
   printf( "  -a, --append             append text to output file\n");
+  printf( "  -b, --block=<n>          process only the specified text block\n");
   printf( "  -C, --copy               'copy' input to output (debug)\n");
-  printf( "  -c, --column=<column>    process only the specified text column\n");
   printf( "  -D, --debug=<level>      (0-100) output intermediate data (debug)\n");
   printf( "  -f, --force              force overwrite of output file\n");
-//  printf( "  -l, --level              level input image before doing OCR\n");
+//  printf( "  -L, --level              level input image before doing OCR\n");
+  printf( "  -l, --layout=<n>         layout analysis, 0=none, 1=column, 2=full\n");
   printf( "  -o <file>                place the output into <file>\n");
-//  printf( "  -r <scale>               reduce input image by <scale> (debug)\n");
-  printf( "  -s <type>                make a 'special file' (debug)\n");
-//  printf( "  -t <threshold>           set threshold for -r option (debug)\n");
+//  printf( "  -R <scale>               reduce input image by <scale> (debug)\n");
+//  printf( "  -s <type>                make a 'special file' (debug)\n");
+//  printf( "  -t <threshold>           set threshold for -R option (debug)\n");
   printf( "  -v, --verbose            be verbose\n");
+  printf( "  -x <file>                export OCR Results File to <file>\n");
 
-  printf( "\nReport bugs to <ant_diaz@teleline.es>.\n");
+  printf( "\nReport bugs to bug-ocrad@gnu.org\n");
   }
 
 
-void ocr( const Blockmap & blockmap, FILE * outfile, int debug_level ) throw()
+void ocr( const Blockmap & blockmap, FILE * outfile, FILE * exportfile,
+          int debug_level ) throw()
   {
   if( debug_level >= 90 )
     { blockmap.print( outfile, debug_level ); return; }
 
-  Textpage textpage( blockmap.block_list() );
+  Textblock textblock( blockmap.block_list() );
 
   // First pass. Recognize the easy characters.
-  textpage.recognize1();
+  textblock.recognize1();
 
   // Second pass. Use context to resolve ambiguities.
-  textpage.recognize2();
+  textblock.recognize2();
 
   if( debug_level >= 86 )
     {
     bool graph = debug_level >= 88 ? true : false;
     bool recursive = (debug_level & 1) ? true : false;
-    textpage.print( outfile, graph, recursive );
+    textblock.print( outfile, graph, recursive );
     return;
     }
 
-  textpage.print( outfile, false, false );
+  textblock.print( outfile, false, false );
+  if( exportfile ) textblock.xprint( exportfile );
   }
 
 
 int main(int argc, char *argv[]) 
   {
-  FILE *infile = stdin, *outfile = stdout;
-  char *outfile_name = 0;
-  int column = -1, debug_level = 0, scale = 0, threshold = -1;
+  FILE *infile = stdin, *outfile = stdout, *exportfile = 0;
+  char *outfile_name = 0, *exportfile_name = 0;
+  int rindex = -1, debug_level = 0, layout_level = 0;
+  int scale = 0, threshold = -1;
   char filetype = 0, specialtype = 0;
   bool append = false, copy = false, force = false, level = false;
   bool verbose = false;
@@ -114,22 +121,23 @@ int main(int argc, char *argv[])
 
   while( true )			// process options
     {
-    int c;
     static struct option long_options[] =
       {
       {"append", no_argument, 0, 'a'},
-      {"column", required_argument, 0, 'c'},
+      {"block", required_argument, 0, 'b'},
       {"copy", no_argument, 0, 'C'},
       {"debug", required_argument, 0, 'D'},
       {"force", no_argument, 0, 'f'},
       {"help", no_argument, 0, 'h'},
-      {"level", no_argument, 0, 'l'},
+      {"layout", required_argument, 0, 'l'},
+      {"level", no_argument, 0, 'L'},
       {"verbose", no_argument, 0, 'v'},
       {"version", no_argument, 0, 'V'},
       {0, 0, 0, 0}
       };
 
-    c = getopt_long( argc, argv, "14aCc:D:fhlo:r:s:t:Vv", long_options, 0 );
+    int c = getopt_long( argc, argv, "14ab:CD:fhLl:o:R:s:t:Vvx:",
+                         long_options, 0 );
     if( c == -1 ) break;		// all options processed
 
     switch( c )
@@ -139,18 +147,20 @@ int main(int argc, char *argv[])
       case '1':
       case '4': filetype = c; break;
       case 'a': append = true; break;
+      case 'b': rindex = strtol( optarg, 0, 0 ) - 1; break;
       case 'C': copy = true; break;
-      case 'c': column = strtol( optarg, 0, 0 ) - 1; break;
       case 'D': debug_level = strtol( optarg, 0, 0 ); break;
       case 'f': force = true; break;
       case 'h': show_help(); return 0;
-      case 'l': level = true; break;
+      case 'L': level = true; break;
+      case 'l': layout_level = strtol( optarg, 0, 0 ); break;
       case 'o':	outfile_name = optarg; break;
-      case 'r': scale = strtol( optarg, 0, 0 ); break;
+      case 'R': scale = strtol( optarg, 0, 0 ); break;
       case 's': specialtype = optarg[0]; break;
       case 't': threshold = strtol( optarg, 0, 0 ); break;
       case 'V':	show_version(); return 0;
       case 'v': verbose = true; break;
+      case 'x':	exportfile_name = optarg; break;
       default: return 1;
       }
     } // end process options
@@ -168,6 +178,15 @@ int main(int argc, char *argv[])
       { fprintf( stderr, "Cannot open %s\n", outfile_name ); return 1; }
     }
 
+  if( exportfile_name )
+    {
+    exportfile = fopen( exportfile_name, "w" );
+    if( !exportfile )
+      { fprintf( stderr, "Cannot open %s\n", exportfile_name ); return 1; }
+    fprintf( exportfile, "# Ocr Results File. Created by GNU %s version %s\n",
+             PROGRAM_NAME, PROGRAM_VERSION );
+    }
+
 // process any remaining command line arguments (input files)
   do {
     if( optind < argc )
@@ -188,18 +207,21 @@ int main(int argc, char *argv[])
       {
       Bitmap page_image( infile );
 
-      // make a list of column coordinates (rectangles)
-      page_image.find_columns( verbose );
+      // make a list of text blocks
+      page_image.analyse_layout( layout_level );
+      if( verbose )
+        fprintf( stderr, "number of text blocks %d\n", page_image.rectangles() );
 
-      if( column >= static_cast< int > ( page_image.rectangle_list().size() ) )
+      if( rindex >= page_image.rectangles() )
         {
-        fprintf( stderr,"This page has only %d column(s)\n",
-                 page_image.rectangle_list().size());
+        fprintf( stderr,"This page has only %d text blocks(s)\n",
+                 page_image.rectangles() );
         return 1;
         }
 
       if( level )
-        fprintf( stderr, "\nslope_best = %d\n", page_image.horizontalify(verbose) );
+        fprintf( stderr, "\nslope_best = %d\n",
+                 page_image.horizontalify( verbose ) );
 
       if( scale != 0 )
         {
@@ -218,21 +240,31 @@ int main(int argc, char *argv[])
         }
       else if( copy )
         {
-        if( column < 0 ) page_image.save( outfile, filetype );
+        if( rindex < 0 ) page_image.save( outfile, filetype );
         else
           {
-          Bitmap bitmap( page_image, page_image.rectangle_list()[column] );
+          Bitmap bitmap( page_image, page_image.rectangle_vector()[rindex] );
           bitmap.save( outfile, filetype );
           }
         return 0;
         }
 
-      // call the character recognizer for every column of text
-      for( int c = 0; c < static_cast<int>(page_image.rectangle_list().size()); ++c )
-        if( column < 0 || column == c )
+      if( exportfile )
+        fprintf( exportfile, "total blocks %d\n",
+                 ( rindex < 0 ) ? page_image.rectangles() : 1 );
+
+      // call the character recognizer for every rectangle of text
+      for( int c = 0; c < page_image.rectangles(); ++c )
+        if( rindex < 0 || rindex == c )
           {
+          if( exportfile )
+            {
+            const Rectangle & r = page_image.rectangle_vector()[c];
+            fprintf( exportfile, "block %d %d %d %d %d\n", c + 1,
+                     r.left(), r.top(), r.width(), r.height() );
+            }
           Blockmap blockmap( page_image, c, debug_level );
-          ocr( blockmap, outfile, debug_level );
+          ocr( blockmap, outfile, exportfile, debug_level );
           }
       }
 
