@@ -1,4 +1,4 @@
-/*  ocrad - Optical Character Recognition program
+/*  GNU OCRAD - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -231,7 +231,8 @@ bool Profile::isconvex() throw()
         { if( data[i] <= 1 ) continue; else return _isconvex; }
       int d = data[i] - data[i-1];
       if( d > dmax ) dmax = d;
-      else if( d < dmax - 2 || ( d < dmax - 1 && d != 0 ) ) return _isconvex;
+      else if( d < dmax - 2 || ( d < dmax - 1 && d != 0 && dmax != 0 ) )
+        return _isconvex;
       }
     _isconvex = true;
     }
@@ -285,18 +286,27 @@ bool Profile::ispit() throw()
     valid_ispit = true; _ispit = false; if( !valid ) initialize();
     if( samples() < 5 ) return _ispit;
 
-    int th = ( mean() < 2 && range() > 2 ) ? 2 : mean();
-    if( data[0] <= th && data[1] <= th ) return _ispit;
-    if( data[samples()-1] <= th && data[samples()-2] <= th ) return _ispit;
-    int status = 0;
-    for( int i = 2; i < samples() - 2; ++i )
-      switch( status )
-        {
-        case 0: if( data[i] < th ) status = 1; break;
-        case 1: if( data[i] > th ) status = 2; break;
-        case 2: if( data[i] < th ) return _ispit;
-        }
-    _ispit = ( status >= 1 );
+    int begin = 0, end = 0, i, ref;
+    for( i = 0, ref = limit(); i < samples(); ++i )
+      {
+      int d = data[i];
+      if( d == min() ) { begin = i; break; }
+      if( d < ref ) ref = d; else if( d > ref + 1 ) return _ispit;
+      }
+    if( begin < 2 || begin > samples() - 3 ) return _ispit;
+
+    for( i = samples() - 1, ref = limit(); i >= begin; --i )
+      {
+      int d = data[i];
+      if( d == min() ) { end = i; break; }
+      if( d < ref ) ref = d; else if( d > ref + 1 ) return _ispit;
+      }
+    if( end < begin || end > samples() - 3 ) return _ispit;
+
+    for( i = begin + 1; i < end; ++i )
+      if( data[i] > min() + 1 ) return _ispit;
+
+    _ispit = true;
     }
   return _ispit;
   }
@@ -345,17 +355,19 @@ bool Profile::isvpit() throw()
     {
     valid_isvpit = true; if( !valid ) initialize();
     if( !ispit() ) { _isvpit = false; return _isvpit; }
+
     int count1 = 0, count2 = 0;
     for( int i = 0; i < samples(); ++i )
       if( data[i] <= 2 ) { if( data[i] < 2 ) ++count1; else ++count2; }
-    _isvpit = ( count1 * 4 < samples() ||
-                ( count1 * 3 < samples() && count2 <= 2 ) );
+    _isvpit = ( 4 * count1 < samples() ||
+                ( 3 * count1 <= samples() && count2 <= 2 ) ||
+                ( 5 * count1 <= 2 * samples() && count2 <= 1 ) );
     }
   return _isvpit;
   }
 
 
-bool Profile::istip() throw()
+bool Profile::istip() throw()		// FIXME
   {
   if( !valid_istip )
     {
@@ -363,14 +375,15 @@ bool Profile::istip() throw()
     if( samples() < 5 ) return _istip;
 
     int th = ( mean() < 2 && range() > 2 ) ? 2 : mean();
-    if( data[0] >= th && data[1] >= th ) return _istip;
-    if( data[samples()-1] >= th && data[samples()-2] >= th ) return _istip;
+    int th2 = ( th > 1 ) ? th : th + 1;
+    if( data[0] >= th2 && data[1] >= th2 ) return _istip;
+    if( data[samples()-1] >= th2 && data[samples()-2] >= th2 ) return _istip;
     int status = 0;
     for( int i = 2; i < samples() - 2; ++i )
       switch( status )
         {
         case 0: if( data[i] > th ) status = 1; break;
-        case 1: if( data[i] < th ) status = 2; break;
+        case 1: if( data[i] <= th ) status = 2; break;
         case 2: if( data[i] > th ) return _istip;
         }
     _istip = ( status >= 1 );
@@ -453,8 +466,8 @@ int Profile::maxima() throw()
 int Profile::minima( int th ) throw()
   {
   if( !valid ) initialize();
-  if( samples() < 1 ) return false;
-  if( th < 2 ) th = ( mean() < 2 ) ? 2 : mean();
+  if( !samples() ) return false;
+  if( th < 1 ) th = ( mean() < 2 ) ? 2 : mean();
   int minima = ( data[0] < th ) ? 1 : 0;
   int status = ( minima ) ? 1 : 0;
 
@@ -473,13 +486,15 @@ bool Profile::straight( int * _dy ) throw()
   if( !valid ) initialize();
   if( samples() < 5 ) return false;
 
-  int xl = 1,             yl = data[1];
-  int xr = samples() - 2, yr = data[samples()-2];
+  int faults = samples() / 10;
+  int xl = ( samples() / 20 ) + 1, yl = data[xl];
+  int xr = samples() - xl - 1,     yr = data[xr];
   int dx = xr - xl, dy = yr - yl;
   for( int i = xl + 1; i < xr; ++i )
     {
     int y = yl; if( dx ) y += ( dy * ( i - xl ) / dx );
-    if( abs( y - data[i] ) > 1 ) return false;
+    int d = abs( y - data[i] );
+    if( d > 2 || ( d > 1 && --faults <= 0 ) ) return false;
     }
   *_dy = dy;
   return true;
