@@ -1,4 +1,4 @@
-/*  ocrad - Optical Character Recognition program
+/*  GNU OCRAD - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -33,7 +33,7 @@
 void Character::recognize1( int charbox_vcenter ) throw()
   {
   if( _block_list.size() == 1 ) recognize11( charbox_vcenter );
-  else if( _block_list.size() == 2 ) recognize12();
+  else if( _block_list.size() == 2 ) recognize12( charbox_vcenter );
   else if( _block_list.size() == 3 ) recognize13();
   }
 
@@ -57,34 +57,47 @@ void Character::recognize110( int charbox_vcenter ) throw()
   {
   const Block & b = _block_list.front();
   Features f( b );
-  unsigned char ch = f.test_solid();
+  unsigned char ch = f.test_solid( charbox_vcenter );
 
-  // FIXME all this mess of commas et all
-  if( !ch && f.test_comma() ) ch = ',';
+  if( !ch && b.top() >= charbox_vcenter && f.test_comma() ) ch = ',';
   if( !ch && b.bottom() <= charbox_vcenter &&
-      2 * b.height() > 3 * b.width() && f.bp.minima() == 1 )
-    { if( f.tp.iminimum() >= f.bp.iminimum() ) ch = '\''; else ch = '`'; }
+      b.height() > b.width() && f.bp.minima() == 1 )
+    {
+    if( f.tp.iminimum() < f.tp.pos(50) && f.bp.iminimum() > f.bp.pos(50) )
+      ch = '`'; else ch = '\'';
+    }
   if( !ch && 2 * b.height() > 3 * b.width() && b.top() >= charbox_vcenter &&
       f.bp.minima() == 1 ) ch = ',';
-  if( ch == ',' && b.bottom() <= charbox_vcenter ) ch = '\'';
-  else if( ch == '\'' )
-    {
-    if( b.top() >= charbox_vcenter ) ch = ',';
-    else if( b.top() + b.width() <= charbox_vcenter &&
-             b.bottom() - b.width() >= charbox_vcenter ) ch = 'I';
-    }
+
   if( ch ) { add_guess( ch, 0 ); return; }
 
   ch = f.test_CEFIJLlT();   if( ch ) { add_guess( ch, 0 ); return; }
-  ch = f.test_frt();        if( ch ) { add_guess( ch, 0 ); return; }
-  ch = f.test_cG();         if( ch ) { add_guess( ch, 0 ); return; }
+  ch = f.test_frst();       if( ch ) { add_guess( ch, 0 ); return; }
+  ch = f.test_G();          if( ch ) { add_guess( ch, 0 ); return; }
+  ch = f.test_c();          if( ch ) { add_guess( ch, 0 ); return; }
   ch = f.test_235Esz();     if( ch ) { add_guess( ch, 0 ); return; }
-  ch = f.test_HKMNUuvwYy(); if( ch ) { add_guess( ch, 0 ); return; }
+  ch = f.test_HKMNUuvwYy();
+  if( ch == 'u' && f.lp.isvpit() )	// Looks for merged 'tr'
+    {
+    int col = b.seek_left( b.vcenter(), b.right() );
+    if( col < b.hpos( 90 ) && !b.escape_top( b.vcenter(), col ) )
+      {
+      col = b.seek_left( b.vcenter(), col - 1, false );
+      while( --col > b.hpos( 40 ) &&
+             ( b.seek_top( b.vcenter(), col ) > b.top() ||
+               f.hp[col-b.left()] > b.height() / 10 ) );
+      if( col > b.hpos( 40 ) )
+        { only_guess( 0, b.left() ); add_guess( 't', col );
+        add_guess( 'r', b.right() ); return; }
+      }
+    }
+  if( ch ) { add_guess( ch, 0 ); return; }
   if( f.bp.minima( b.height() / 8 ) == 2 )
     {
     if( f.tp.minima( b.height() / 8 ) == 2 && f.lp.istip() && f.rp.istip() )
       { add_guess( 'x', 0 ); return; }
-    ch = f.test_hnw(); if( ch == 'n' )	// Looks for merged 'rr' or 'fl'
+    ch = f.test_hnw();
+    if( ch == 'n' )	// Looks for merged 'rr' or 'fl'
       {
       if( b.width() > b.height() )
         {
@@ -110,6 +123,19 @@ void Character::recognize110( int charbox_vcenter ) throw()
           }
         }
       }
+    if( ch == 'h' )	// Looks for merged 'rf'
+      {
+      if( b.seek_right( b.top() + 1, b.left() ) > b.hcenter() )
+        {
+        int col = 0, hmin = f.hp.range() + 1;
+        for( int i = b.hpos(40); i <= b.hpos(60); ++i )
+          if( f.hp[i-b.left()] < hmin )
+            { hmin = f.hp[i-b.left()]; col = i; }
+        if( col > b.left() && col < b.right() )
+          { only_guess( 0, b.left() ); add_guess( 'r', col - 1 );
+          add_guess( 'f', b.right() ); return; }
+        }
+      }
     if( ch ) { add_guess( ch, 0 ); return; }
     }
   if( f.bp.minima() == 3 ) { add_guess( 'm', 0 ); return; }
@@ -129,12 +155,8 @@ void Character::recognize110( int charbox_vcenter ) throw()
   if( f.rp.isconvex() && b.height() > 2 * b.width() )
     { add_guess( ')', 0 ); return; }
 
-  ch = f.test_line();
-  if( ch )
-    {
-    if( ch == '\'' && b.top() >= charbox_vcenter ) ch = ',';
-    add_guess( ch, 0 ); return;
-    }
+  ch = f.test_line( charbox_vcenter );
+  if( ch ) { add_guess( ch, 0 ); return; }
 
   ch = f.test_misc();
   if( ch ) { add_guess( ch, 0 ); return; }
@@ -149,33 +171,42 @@ void Character::recognize111() throw()
   const Block & h = b.block_list().front();
   Features f( b );
 //  Features fh( h );
+  int top_delta = h.top() - b.top(), bottom_delta = b.bottom() - h.bottom();
 
-  if( similar( h.top() - b.top(), b.bottom() - h.bottom(), 40 ) )
+  if( abs( top_delta - bottom_delta ) <= 2 ||
+      similar( top_delta, bottom_delta, 40 ) )
     {
     // hole is vertically centered
     if( f.lp.isflats() && !f.rp.isflats() ) { add_guess( 'D', 0 ); return; }
-    if( !f.lp.isconvex() && !f.rp.isconvex() )
+    if( !f.rp.isconvex() )
       {
       if( f.tp.minima() == 2 ) { add_guess( '#', 0 ); return; }
       if( f.tp.minima() == 1 && f.bp.minima() == 1 )
         {
-        if( f.bp.isconvex() )
+        int row = b.seek_bottom( h.bottom(), h.hcenter(), false );
+        if( !f.lp.isconvex() && f.bp.isconvex() &&
+            b.seek_bottom( row, h.hcenter() ) < b.bottom() )
           { add_guess( ISO_8859_1::SEACUTE, 0 ); return; }
-        else if( !f.lp.isflats() ) { add_guess( '4', 0 ); return; }
+        row = ( row + b.seek_bottom( row, h.hcenter() ) ) / 2;
+        if( !f.lp.isflats() && b.seek_left( row, h.hcenter() ) <= b.left() )
+          { add_guess( '4', 0 ); return; }
         }
-      if( f.bp.minima() == 2 ) { add_guess( 'A', 0 ); return; }
+      if( b.width() > 3 * h.width() )
+        {
+        int c = f.vscan()[b.vcenter() - b.top()];
+        if( c == 3 && !f.lp.isconvex() ) { add_guess( 'm', 0 ); return; }
+        if( c >= 4 ) { add_guess( '@', 0 ); return; }
+        }
+      if( f.tp.minima() == 1 && f.bp.minima() == 2 && !f.lp.isconvex() )
+        { add_guess( 'A', 0 ); return; }
       }
     if( similar( h.left() - b.left(), b.right() - h.right(), 40 ) )
       {
-      if( b.width() > 3 * h.width() && f.vscan()[b.vcenter() - b.top()] == 4 )
-        { add_guess( '@', 0 ); return; }
-//      if( h.block_list().size() ) { add_guess( '0', 0 ); return; }
-//      if( f.lp.ispit() )
-        {
-        if( b.width() >= b.height() || similar( b.height(), b.width(), 21 ) )
-          { add_guess( 'o', 0 ); return; }
-        add_guess( 'O', 0 ); return;
-        }
+      if( !f.lp.isconvex() && !f.bp.isconvex() && !f.rp.isflat() &&
+          f.bp[1] < 2 ) { add_guess( 'D', 0 ); return; }
+      if( f.bp.minima() > 1 || f.rp.minima() > 1 )
+        { add_guess( 'a', 1 ); add_guess( 'Q', 0 ); return; }
+      add_guess( 'o', 0 ); return;
       }
     if( h.left() - b.left() > b.right() - h.right() && f.rp.ispit() )
       { add_guess( 'D', 0 ); return; }
@@ -183,7 +214,7 @@ void Character::recognize111() throw()
       { add_guess( 'a', 0 ); return; }
     return;
     }
-  if( h.top() - b.top() < b.bottom() - h.bottom() )
+  if( top_delta < bottom_delta )
     {
     // hole is high
     if( f.bp.minima( b.height() / 10 ) == 2 && f.bp.isctip() )
@@ -194,7 +225,7 @@ void Character::recognize111() throw()
     unsigned char ch = f.test_49egpq();
     if( ch ) { add_guess( ch, 0 ); return; }
     }
-  if( h.top() - b.top() > b.bottom() - h.bottom() )
+  if( top_delta > bottom_delta )
     {
     // hole is low
     unsigned char ch = f.test_6abd();
@@ -211,6 +242,9 @@ void Character::recognize111() throw()
         }
       return;
       }
+    if( b.width() > 3 * h.width() && f.vscan()[b.vcenter() - b.top()] == 3 &&
+        !f.lp.isconvex() )
+      { add_guess( 'm', 0 ); return; }
     }
   }
 
@@ -233,6 +267,11 @@ void Character::recognize112() throw()
   Profile bp( b, Profile::bottom );
   if( similar( a1, a2, 50 ) )		// I don't like this
     {
+    int hdiff;
+    if( b.bottom_hook( &hdiff ) && hdiff > b.height() / 2 )
+      if( b.top_hook( &hdiff ) && hdiff > b.height() / 2 )
+        { add_guess( 's', 0 ); return; }
+  
     if( lp.isflats() ) { add_guess( 'B', 0 ); return; }
 
     int col1 = h1.seek_left( h1.bottom(), h1.right() + 1 ) - 1;
@@ -247,14 +286,18 @@ void Character::recognize112() throw()
     if( bp.isconvex() ) { add_guess( '8', 0 ); return; }
     add_guess( 'B', 0 ); return;
     }
-  if( a1 > a2 ) { add_guess( 'Q', 0 ); return; }
+  if( a1 > a2 )
+    {
+    if( !h1.v_overlaps( h2 ) ) { add_guess( 'g', 0 ); return; }
+    add_guess( 'Q', 0 ); return;
+    }
   add_guess( '&', 0 );
   }
 
 
 // Recognizes 2 block characters.
 // ij!%:;=?|
-void Character::recognize12() throw()
+void Character::recognize12( int charbox_vcenter ) throw()
   {
   const Block & b1 = _block_list.front();		// lower block
   const Block & b2 = _block_list.back();		// upper block
@@ -272,17 +315,25 @@ void Character::recognize12() throw()
     }
   if( similar( a1, a2, 60 ) )
     {
-    if( f2.test_solid() == '.' && b1.height() > b2.height() )
-      { add_guess( ';', 0 ); return; }
-    unsigned char ch = f1.test_solid();
+    if( f2.test_solid( charbox_vcenter ) == '.' )
+      {
+      if( f1.test_solid( charbox_vcenter ) == '.' )
+        { add_guess( ':', 0 ); return; }
+      if( b1.height() > b2.height() ) { add_guess( ';', 0 ); return; }
+      }
+    unsigned char ch = f1.test_solid( charbox_vcenter );
     if( ch == '-' || ch == '_' ) { add_guess( '', 0 ); return; }
+    if( b1.includes_hcenter( b2 ) && b2.includes_hcenter( b1 ) )
+      { if( b1.blocks() && b2.blocks() ) add_guess( 'g', 0 ); else return; }
     add_guess( '%', 0 ); return;
     }
   if( a1 > a2 )
     {
-    unsigned char ch = f2.test_solid();
-    if( ch == '.' || ch == '\'' )
+    unsigned char ch = f2.test_solid( charbox_vcenter );
+    if( ch == '-' && 2 * b2.height() > b2.width() ) ch = '.';	//FIXME
+    if( !b1.blocks() && ( ch == '.' || ch == '\'' ) )
       {
+      // Looks for merged 'ri'
       if( f1.bp.minima( b1.height() / 4 ) == 2 &&
           b1.top() > b2.bottom() && b1.hcenter() < b2.left() )
         {
@@ -300,29 +351,37 @@ void Character::recognize12() throw()
           }
         }
       if( f1.bp.minima( b1.height() / 4 ) != 1 ) return;
-      int hdiff;
-      if( b1.bottom_hook( &hdiff ) )
+
         {
-        if( abs( hdiff ) < b1.height() / 2 ) return;
-        if( hdiff > 0 && f1.rp.increasing( f1.rp.pos( 75 ) ) )
-          { add_guess( 'j', 0 ); return; }
-        if( hdiff < 0 )
+        int hdiff;
+        if( b1.bottom_hook( &hdiff ) )
           {
-          if( f1.wp.max() > 2 * f2.wp.max() && f1.lp.minima() == 1 )
-            { add_guess( '', 0 ); return; }
-          add_guess( 'i', 0 ); return;
+          if( abs( hdiff ) < b1.height() / 2 ) return;
+          if( hdiff > 0 && f1.rp.increasing( f1.rp.pos( 75 ) ) )
+            { add_guess( 'j', 0 ); return; }
+          if( hdiff < 0 )
+            {
+            if( f1.wp.max() > 2 * f2.wp.max() && f1.lp.minima() == 1 )
+              { add_guess( ISO_8859_1::IQUEST, 0 ); return; }
+            add_guess( 'i', 0 ); return;
+            }
           }
         }
-      if( similar( f1.wp.max(), f2.wp.max(), 20 ) && f1.wp[1] < f2.wp.max() )
-        { add_guess( '', 0 ); return; }
-      if( b1.seek_bottom( b1.vcenter(), b1.hpos( 25 ) ) < b1.bottom() &&
+      if( similar( f1.wp.max(), f2.wp.max(), 20 ) )
+        {
+        if( 3 * f1.wp[f1.wp.pos(10)] < 2 * f2.wp.max() )
+          { add_guess( '', 0 ); return; }
+        add_guess( 'i', 0 ); return;
+        }
+      if( 3 * f1.wp.max() > 4 * f2.wp.max() &&
+          b1.seek_bottom( b1.vcenter(), b1.hpos( 10 ) ) < b1.bottom() &&
           f1.rp.increasing( f1.rp.pos( 75 ) ) )
         { add_guess( 'j', 0 ); return; }
       add_guess( 'i', 0 ); return;
       }
     int slope;
     unsigned char atype = '\'';
-    if( f2.tp.ispit() ) atype = '^';
+    if( f2.bp.minima() == 2 ) atype = '^';
     else if( f2.rp.straight( &slope ) && slope < 0 ) atype = '`';
     Character c( b1 );
     c.recognize1( c.vcenter() );
@@ -331,7 +390,7 @@ void Character::recognize12() throw()
     if( ch ) add_guess( ch, 0 ); else add_guess( atype , 0 );
     return;
     }
-  unsigned char ch = f1.test_solid();
+  unsigned char ch = f1.test_solid( charbox_vcenter );
   if( ch == '.' )
     {
     if( similar( b1.width(), b2.width(), 50 ) ) { add_guess( '!', 0 ); return; }
@@ -356,7 +415,7 @@ void Character::recognize13() throw()
   {
   const Block & b1 = _block_list.front();
   Features f1( b1 );
-  if( f1.test_solid() == '.' ) add_guess( '', 0 );
+  if( f1.test_solid( vcenter() ) == '.' ) add_guess( '', 0 );
   Character c( b1 );
   c.recognize1( c.vcenter() );
   if( c.guesses() )
@@ -370,6 +429,7 @@ void Character::recognize13() throw()
       case 'U': add_guess( '', 0 ); return;
       case 'a': add_guess( '', 0 ); return;
       case 'e': add_guess( '', 0 ); return;
+      case '|':
       case 'l':
       case 'i': add_guess( '', 0 ); return;
       case 'o': add_guess( '', 0 ); return;
