@@ -1,4 +1,4 @@
-/*  ocrad - Optical Character Recognition program
+/*  GNU OCRAD - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -17,6 +17,7 @@
 */
 
 #include <cstddef>
+#include <cstdio>
 #include "common.h"
 #include "rectangle.h"
 
@@ -24,7 +25,10 @@
 Rectangle::Rectangle( int l, int t, int r, int b ) throw()
   {
   if( r < l || b < t )
+    {
+    fprintf( stderr, "l = %d, t = %d, r = %d, b = %d\n", l, t, r, b );
     internal_error( "bad parameter building a Rectangle" );
+    }
   _left = l; _top = t; _right = r; _bottom = b;
   }
 
@@ -72,10 +76,10 @@ void Rectangle::add_point( int row, int col ) throw()
 
 void Rectangle::add_rectangle( const Rectangle & r ) throw()
   {
-  if( r._bottom > _bottom ) _bottom = r._bottom;
+  if( r._left < _left ) _left = r._left;
   if( r._top < _top ) _top = r._top;
   if( r._right > _right ) _right = r._right;
-  if( r._left < _left ) _left = r._left;
+  if( r._bottom > _bottom ) _bottom = r._bottom;
   }
 
 bool Rectangle::includes( const Rectangle & r ) const throw()
@@ -84,6 +88,11 @@ bool Rectangle::includes( const Rectangle & r ) const throw()
            _right >= r._right && _bottom >= r._bottom );
   }
 
+bool Rectangle::includes( int row, int col ) const throw()
+  {
+  return ( _left <= col && _right >= col && _top <= row && _bottom >= row );
+  }
+
 bool Rectangle::includes_hcenter( const Rectangle & r ) const throw()
   { return ( _left <= r.hcenter() && _right >= r.hcenter() ); }
 
@@ -96,6 +105,12 @@ bool Rectangle::h_includes( const Rectangle & r ) const throw()
 bool Rectangle::v_includes( const Rectangle & r ) const throw()
   { return ( _top <= r._top && _bottom >= r._bottom ); }
 
+bool Rectangle::h_includes( int col ) const throw()
+  { return ( _left <= col && _right >= col ); }
+
+bool Rectangle::v_includes( int row ) const throw()
+  { return ( _top <= row && _bottom >= row ); }
+
 bool Rectangle::h_overlaps( const Rectangle & r ) const throw()
   { return ( _left <= r.right() && _right >= r.left() ); }
 
