@@ -1,5 +1,19 @@
-/*  ocrad - Optical Character Recognition program
+/*  GNU OCRAD - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
+
+    This program is free software; you can redistribute it and/or modify
+    it under the terms of the GNU General Public License as published by
+    the Free Software Foundation; either version 2 of the License, or
+    (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public License
+    along with this program; if not, write to the Free Software
+    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
 
 class Rectangle
@@ -31,10 +45,13 @@ public:
   void add_rectangle( const Rectangle & r ) throw();
 
   bool includes( const Rectangle & r ) const throw();
+  bool includes( int row, int col ) const throw();
   bool includes_hcenter( const Rectangle & r ) const throw();
   bool includes_vcenter( const Rectangle & r ) const throw();
   bool h_includes( const Rectangle & r ) const throw();
+  bool h_includes( int col ) const throw();
   bool v_includes( const Rectangle & r ) const throw();
+  bool v_includes( int row ) const throw();
   bool h_overlaps( const Rectangle & r ) const throw();
   bool v_overlaps( const Rectangle & r ) const throw();
   };
