@@ -1,4 +1,4 @@
-/*  ocrad - Optical Character Recognition program
+/*  GNU OCRAD - Optical Character Recognition program
     Copyright (C) 2003 Antonio Diaz Diaz.
 
     This program is free software; you can redistribute it and/or modify
@@ -29,9 +29,9 @@
 #include "textline.h"
 
 
-void Textline::add_character( const Character & c ) throw()
+const Character & Textline::add_character( const Character & c ) throw()
   {
-  data.push_back( c );
+  data.push_back( c ); return data.back();
   }
 
 
@@ -51,6 +51,17 @@ void Textline::delete_character( int i ) throw()
   }
 
 
+const Character & Textline::shift_character( const Character & c ) throw()
+  {
+  int i;
+
+  for( i = characters(); i > 0; --i )
+    if( c.left() >= data[i-1].left() ) break;
+  data.insert( data.begin() + i, c );
+  return data[i];
+  }
+
+
 void Textline::swap_characters( int i, int j ) throw()
   {
   if( i < 0 || i >= characters() || j < 0 || j >= characters() )
@@ -77,6 +88,16 @@ int Textline::mean_height() const throw()
   }
 
 
+int Textline::mean_height1() const throw()
+  {
+  int sum = 0;
+
+  if( characters() <= 1 ) return sum;
+  for( int i = 1; i < characters(); ++i ) sum += data[i].height();
+  return sum / ( characters() - 1 );
+  }
+
+
 int Textline::mean_width() const throw()
   {
   int sum = 0;
@@ -89,14 +110,13 @@ int Textline::mean_width() const throw()
 
 int Textline::mean_gap_width() const throw()
   {
-  int sum = 0, gaps = 0;
+  int sum = 0;
 
   if( characters() < 2 ) return sum;
   for( int i = 1; i < characters(); ++i )
     if( data[i].left() > data[i-1].right() )
-      { ++gaps; sum += data[i].left() - data[i-1].right() - 1; }
-  if( gaps ) sum /= gaps;
-  return sum;
+      sum += data[i].left() - data[i-1].right() - 1;
+  return sum / ( characters() - 1 );
   }
 
 
@@ -138,6 +158,13 @@ void Textline::print( FILE * outfile, bool graph, bool recursive ) const throw()
   }
 
 
+void Textline::xprint( FILE * exportfile ) const throw()
+  {
+  for( int i = 0; i < characters(); ++i )
+    character(i).xprint( exportfile );
+  }
+
+
 void Textline::recognize1() const throw()
   {
   if( characters() == 0 ) return;
@@ -239,13 +266,30 @@ void Textline::recognize2() throw()
           if( isspace( ch2 ) ) break;
           if( ( isalpha( ch2 ) && 4 * c1.height() > 5 * c2.height() ) ||
               ( isupper( ch2 ) && ch2 != 'B' &&
-                similar( c1.height(), c2.height(), 10 ) ) )
+                ( c1.height() >= c2.height() ||
+                  similar( c1.height(), c2.height(), 10 ) ) ) )
             { c1.insert_guess( 0, toupper( ch ), 1 ); break; }
           }
         }
       }
     }
 
+  // transform 'i' into 'j'
+  for( int i = 0; i < characters(); ++i )
+    {
+    Character & c1 = character( i );
+    if( c1.guesses() == 1 && c1.guess( 0 ).ch == 'i' )
+      {
+      int j = i + 1;
+      if( j >= characters() || !character( j ).guesses() )
+        { j = i - 1; if( j < 0 || !character( j ).guesses() ) continue; }
+      Character & c2 = character( j );
+      if( ISO_8859_1::isvowel( c2.guess( 0 ).ch ) &&
+          c1.bottom() >= c2.bottom() + ( c2.height() / 4 ) )
+        c1.insert_guess( 0, 'j', 1 );
+      }
+    }
+
   // transform a small p to a capital p
   for( int i = characters() - 1; i > 0; --i )
     {
@@ -296,6 +340,25 @@ void Textline::recognize2() throw()
       }
     }
 
+  // transform a vertical bar into l or I
+  for( int i = 0; i < characters(); ++i )
+    {
+    Character & c = character( i );
+    if( c.guesses() == 1 && c.guess( 0 ).ch == '|' )
+      {
+      unsigned char lch = 0, rch = 0;
+      if( i > 0 && character( i - 1 ).guesses() )
+        lch = character( i - 1 ).guess( 0 ).ch;
+      if( i < characters() - 1 && character( i + 1 ).guesses() )
+        rch = character( i + 1 ).guess( 0 ).ch;
+      if( ISO_8859_1::isupper( rch ) &&
+          ( !lch || ISO_8859_1::isupper( lch ) || isspace( lch ) ) )
+        { c.insert_guess( 0, 'I', 1 ); continue; }
+      if( ISO_8859_1::isalpha( lch ) || ISO_8859_1::isalpha( rch ) )
+        { c.insert_guess( 0, 'l', 1 ); continue; }
+      }
+    }
+
   // join two adjacent single quotes into a double quote
   for( int i = 0; i < characters() - 1; ++i )
     {
@@ -334,7 +397,7 @@ void Textline::recognize2() throw()
       {
       unsigned char ch = c1.guess( 0 ).ch;
       if( isspace( ch ) ) { begin = i + 1 ; continue; }
-      if( ch != 'O' && ch != 'l' ) continue;
+      if( ch != 'o' && ch != 'O' && ch != 'l' ) continue;
       for( int j = begin; j < characters(); ++j ) if( j != i )
         {
         Character & c2 = character( j );
